package com.mongo.demo.service;

import com.mongo.demo.dto.EmployeeDto;
import com.mongo.demo.entity.Employee;
import com.mongo.demo.exception.EmployeeNotFoundException;
import com.mongo.demo.repositery.EmployeeRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mongo.demo.utils.EmployeeMapper;
import reactor.core.publisher.Mono;

import java.util.List;

@Service
public class EmployeeService {

    @Autowired
    private EmployeeRepository employeeRepository;

    public Mono<List<Employee>> getEmployeesDetails(){
        return employeeRepository.findAll().collectList();
    }

    public Mono<EmployeeDto> getEmployeeDetails(String id){
        Mono<Employee> byId = employeeRepository.findById(id).switchIfEmpty(Mono.error(new EmployeeNotFoundException("Employee with ID " + id + " not found")));
        return  byId.flatMap(employee -> {
            return  Mono.just(EmployeeMapper.entityToDto(employee));
        });
    }

    public  Mono<EmployeeDto> createEmployee(Mono<EmployeeDto> employeeDtoMono){
        return  employeeDtoMono
                .map(EmployeeMapper::dtoToEntity)
                .flatMap(employeeRepository :: insert)
                .map(EmployeeMapper :: entityToDto);
    }

    public Mono<String> deleteEmployee(String id){
        return employeeRepository.existsById(id)
                .flatMap(exists -> {
                    if (exists) {
                        return employeeRepository.deleteById(id)
                                .then(Mono.just("Success"));
                    } else {
                        return Mono.just("No record found");
                    }
                });
    }
}
